/*============================================================================
  File:     TransparentDataEncryption.sql

  Summary:  Show various facets of TDE

  Date:     March 2008

  SQL Server Version: 10.0.1300.13 (Katmai February CTP - CTP-6)
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp & Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended as a supplement to the SQL Server 2008 Jumpstart or
  Metro training.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Two copies of the same database - one is encrypted. What's the
-- performance difference?

-- Make the buffer pool cold
DBCC DROPCLEANBUFFERS;
GO

-- Do a count of rows in the biggest table - requires reading
-- every page
SELECT COUNT (*) FROM SalesDB.dbo.Sales;
GO
-- Approx 8-9 seconds

-- How big is the table?
SELECT [page_count] FROM sys.dm_db_index_physical_stats (
	DB_ID ('SalesDB'), OBJECT_ID ('SalesDB.dbo.Sales'),
	NULL, NULL, 'LIMITED');
GO

-- Now do the same count on the encrypted copy of the database
DBCC DROPCLEANBUFFERS;
GO
SELECT COUNT (*) FROM SalesDBEncrypted.dbo.Sales;
GO

-- What's the difference?

-- Reads take about the same. What about writes???

-- First the unencrypted data
DBCC DROPCLEANBUFFERS;
GO

UPDATE SalesDB.dbo.Sales
	SET Quantity = 3
	WHERE SalesID < 500000;
GO
CHECKPOINT
GO
-- Approx 17 seconds

-- Next the encrypted data
DBCC DROPCLEANBUFFERS;
GO

UPDATE SalesDBEncrypted.dbo.Sales
	SET Quantity = 3
	WHERE SalesID < 500000;
GO
CHECKPOINT
GO

-- What's the difference?

-- Create a test database, with a table with 'secure' data
IF DB_ID ('MySecureData') IS NOT NULL
	DROP DATABASE MySecureData;
GO

CREATE DATABASE MySecureData;
GO
CREATE TABLE MySecureData.dbo.MySecureTable (
	CustomerID INT IDENTITY,
	Name		VARCHAR (100) NOT NULL,
	CCNumber	VARCHAR (16) NOT NULL);
GO

INSERT INTO MySecureData.dbo.MySecureTable VALUES (
	'Paul Randal', '4444555566667777');
INSERT INTO MySecureData.dbo.MySecureTable VALUES (
	'Kimberly Tripp', '4444888899990000');
GO

-- Backup the database
BACKUP DATABASE MySecureData TO
DISK = N'C:\SQLskills\MySecureData.bak'
WITH INIT;
GO

-- Restore another copy
RESTORE DATABASE MySecureDataEncrypted 
FROM DISK = N'C:\SQLskills\MySecureData.bak' WITH
MOVE 'MySecureData' TO
	N'C:\Program Files\Microsoft SQL Server\MSSQL10.SQLDEV01\MSSQL\DATA\MySecureDataEncrypted.mdf',
MOVE 'MySecureData_log' TO
	N'C:\Program Files\Microsoft SQL Server\MSSQL10.SQLDEV01\MSSQL\DATA\MySecureDataEncrypted_log.ldf',
REPLACE;
GO

-- Encrypt the database copy
--CREATE MASTER KEY ENCRYPTION BY
--PASSWORD = 'Sligachan*01*';
--GO
-- There's already a master key

CREATE CERTIFICATE MyServerCert
WITH SUBJECT = 'My DEK Certificate';
GO

USE MySecureDataEncrypted;
GO

CREATE DATABASE ENCRYPTION KEY
WITH ALGORITHM = AES_128
ENCRYPTION BY SERVER CERTIFICATE MyServerCert;
GO

ALTER DATABASE MySecureDataEncrypted
SET ENCRYPTION ON;
GO

-- See if encryption has completed yet
SELECT * FROM sys.dm_database_encryption_keys
WHERE [database_id] = DB_ID ('MySecureDataEncrypted');
GO
-- Encryption state of 3 means it's encrypted, 2 means
-- encryption is still in progress

-- Let's backup the database so we can use it somewhere else
BACKUP DATABASE MySecureDataEncrypted
TO DISK = 'C:\SQLskills\MySecureDataEncrypted.bak'
WITH INIT;

-- And backup the server certificate
USE master;
GO
BACKUP CERTIFICATE MyServerCert
TO FILE = 'C:\SQLskills\MyServerCert'
WITH PRIVATE KEY (FILE = 'C:\SQLskills\MyServerCertKey',
	ENCRYPTION BY PASSWORD = 'PSRxKLT@07292007');
GO

-- Now that encryption has completed, shutdown the server
SHUTDOWN WITH NOWAIT;
GO

-- Look in C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data



